@@ -1,3 +1,5 @@
+//go:generate mockgen -source=$GOFILE -destination=mock_$GOFILE -package=$GOPACKAGE Storage
+
 // Package storage implements operations for s3 and fs.
 package storage
 
@@ -11,9 +13,9 @@ import (
 	"time"
 
 	"github.com/lanrat/extsort"
-	"github.com/peak/s5cmd/log"
-	"github.com/peak/s5cmd/storage/url"
-	"github.com/peak/s5cmd/strutil"
+	"github.com/peak/s5cmd/v2/log"
+	"github.com/peak/s5cmd/v2/storage/url"
+	"github.com/peak/s5cmd/v2/strutil"
 )
 
 var (
@@ -166,10 +168,15 @@ func (o ObjectType) IsSymlink() bool {
 	return o.mode&os.ModeSymlink != 0
 }
 
-// ShouldProcessUrl returns true if follow symlinks is enabled.
+// IsRegular checks if the object is a regular file.
+func (o ObjectType) IsRegular() bool {
+	return o.mode.IsRegular()
+}
+
+// ShouldProcessURL returns true if follow symlinks is enabled.
 // If follow symlinks is disabled we should not process the url.
 // (this check is needed only for local files)
-func ShouldProcessUrl(url *url.URL, followSymlinks bool) bool {
+func ShouldProcessURL(url *url.URL, followSymlinks bool) bool {
 	if followSymlinks {
 		return true
 	}
@@ -212,83 +219,18 @@ func (s StorageClass) IsGlacier() bool {
 	return s == "GLACIER"
 }
 
-type Metadata map[string]string
+type Metadata struct {
+	ACL                string
+	CacheControl       string
+	Expires            string
+	StorageClass       string
+	ContentType        string
+	ContentDisposition string
+	EncryptionMethod   string
+	EncryptionKeyID    string
+	ContentEncoding    string
 
-// NewMetadata will return an empty metadata object.
-func NewMetadata() Metadata {
-	return Metadata{}
-}
-
-func (m Metadata) ACL() string {
-	return m["ACL"]
-}
-
-func (m Metadata) SetACL(acl string) Metadata {
-	m["ACL"] = acl
-	return m
-}
-
-func (m Metadata) CacheControl() string {
-	return m["CacheControl"]
-}
-
-func (m Metadata) SetCacheControl(cacheControl string) Metadata {
-	m["CacheControl"] = cacheControl
-	return m
-}
-
-func (m Metadata) Expires() string {
-	return m["Expires"]
-}
-
-func (m Metadata) SetExpires(expires string) Metadata {
-	m["Expires"] = expires
-	return m
-}
-
-func (m Metadata) StorageClass() string {
-	return m["StorageClass"]
-}
-
-func (m Metadata) SetStorageClass(class string) Metadata {
-	m["StorageClass"] = class
-	return m
-}
-
-func (m Metadata) ContentType() string {
-	return m["ContentType"]
-}
-
-func (m Metadata) SetContentType(contentType string) Metadata {
-	m["ContentType"] = contentType
-	return m
-}
-
-func (m Metadata) SSE() string {
-	return m["EncryptionMethod"]
-}
-
-func (m Metadata) SetSSE(sse string) Metadata {
-	m["EncryptionMethod"] = sse
-	return m
-}
-
-func (m Metadata) SSEKeyID() string {
-	return m["EncryptionKeyID"]
-}
-
-func (m Metadata) SetSSEKeyID(kid string) Metadata {
-	m["EncryptionKeyID"] = kid
-	return m
-}
-
-func (m Metadata) ContentEncoding() string {
-	return m["ContentEncoding"]
-}
-
-func (m Metadata) SetContentEncoding(contentEncoding string) Metadata {
-	m["ContentEncoding"] = contentEncoding
-	return m
+	UserDefined map[string]string
 }
 
 func (o Object) ToBytes() []byte {
@@ -304,9 +246,9 @@ func (o Object) ToBytes() []byte {
 
 func FromBytes(data []byte) extsort.SortType {
 	dec := gob.NewDecoder(bytes.NewBuffer(data))
-	var gobUrl []byte
-	dec.Decode(&gobUrl)
-	u := url.FromBytes(gobUrl).(*url.URL)
+	var gobURL []byte
+	dec.Decode(&gobURL)
+	u := url.FromBytes(gobURL).(*url.URL)
 	o := Object{
 		URL: u,
 	}
